// ELF compatibility interpreter

#include <linux/interpreter.h>
#include <linux/module.h>

#if defined(__x86_64__)
#define INTERPRETER_I386_LINUX_GNU
static char *interpreter_i386_linux_gnu = "/lib/i386-linux-gnu/ld-linux.so.2";
module_param(interpreter_i386_linux_gnu, charp, 0444);
MODULE_PARM_DESC(interpreter_i386_linux_gnu,
	"ELF compatibility interpreter for i386-linux-gnu");
#endif

#if defined(__x86_64__)
#define INTERPRETER_X86_64_LINUX_GNU
static char *interpreter_x86_64_linux_gnu
	= "/lib/x86_64-linux-gnu/ld-linux-x86-64.so.2";
module_param(interpreter_x86_64_linux_gnu, charp, 0444);
MODULE_PARM_DESC(interpreter_x86_64_linux_gnu,
	"ELF compatibility interpreter for x86_64-linux-gnu");
#endif

#if defined(__x86_64__)
#define INTERPRETER_X86_64_LINUX_GNUX32
static char *interpreter_x86_64_linux_gnux32
	= "/lib/x86_64-linux-gnux32/ld-linux-x32.so.2";
module_param(interpreter_x86_64_linux_gnux32, charp, 0444);
MODULE_PARM_DESC(interpreter_x86_64_linux_gnux32,
	"ELF compatibility interpreter for x86_64-linux-gnux32");
#endif

static const char *elf_compat_alternative_name(const char *name) {
	return
#ifdef INTERPRETER_I386_LINUX_GNU
		!strcmp(name, "/lib/ld-linux.so.2")? interpreter_i386_linux_gnu:
#endif
#ifdef INTERPRETER_X86_64_LINUX_GNU
		!strcmp(name, "/lib64/ld-linux-x86-64.so.2")?
		interpreter_x86_64_linux_gnu:
#endif
#ifdef INTERPRETER_X86_64_LINUX_GNUX32
		!strcmp(name, "/libx32/ld-linux-x32.so.2")?
		interpreter_x86_64_linux_gnux32:
#endif
		NULL;
}

static struct linux_interpreter elf_compat_interpreter = {
	.module = THIS_MODULE,
	.alternative_name = elf_compat_alternative_name
};

static int __init init_elf_compat_interpreter(void) {
	register_interpreter(&elf_compat_interpreter);
	return 0;
}

static void __exit exit_elf_compat_interpreter(void) {
	unregister_interpreter(&elf_compat_interpreter);
}

core_initcall(init_elf_compat_interpreter);
module_exit(exit_elf_compat_interpreter);
MODULE_LICENSE("GPL");
